// $Id$
/**
* Copyright (C) 2017 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.descriptiveDataSet;

import java.util.Collection;
import java.util.List;
import java.util.Map;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.inject.Inject;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.services.EMenuService;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.e4.ui.workbench.modeling.ESelectionService;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IMemento;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.api.service.IDescriptiveDataSetService;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.description.DescriptiveDataSet;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.model.IContextListener;
import eu.etaxonomy.taxeditor.session.ICdmEntitySession;
import eu.etaxonomy.taxeditor.session.ICdmEntitySessionEnabled;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author pplitzner
 * @date 12.12.2017
 *
 */
public class DescriptiveDataSetNavigator implements IConversationEnabled, ICdmEntitySessionEnabled, IContextListener{

    private ListViewer viewer;

    @Inject
    private ESelectionService selService;

    private ISelectionChangedListener selectionChangedListener;

    private ConversationHolder conversation;

    private ICdmEntitySession cdmEntitySession;

    private List<DescriptiveDataSet> descriptiveDataSets;

    public DescriptiveDataSetNavigator() {
        CdmStore.getContextManager().addContextListener(this);
    }

    @PostConstruct
    public void create(Composite parent, EPartService partService, EModelService modelService, MApplication application,
            EMenuService menuService){
        viewer = new ListViewer(parent);
        viewer.setContentProvider(new ArrayContentProvider());
        viewer.setLabelProvider(new LabelProvider(){
            /**
             * {@inheritDoc}
             */
            @Override
            public String getText(Object element) {
                if(element instanceof DescriptiveDataSet){
                    return ((DescriptiveDataSet) element).getLabel();
                }
                return super.getText(element);
            }
        });

        viewer.addDoubleClickListener(new IDoubleClickListener() {

            @Override
            public void doubleClick(DoubleClickEvent event) {
                ISelection selection = event.getSelection();
                if(selection instanceof IStructuredSelection){
                	DescriptiveDataSet descriptiveDataSetUuid = (DescriptiveDataSet) ((IStructuredSelection) selection).getFirstElement();
                	if(descriptiveDataSetUuid!=null){
                	    EditorUtil.openDescriptiveDataSetEditor(descriptiveDataSetUuid.getUuid(), modelService, partService, application);
                	}
                }
            }
        });

        //propagate selection
       selectionChangedListener = (event -> selService.setSelection(event.getSelection()));
       viewer.addSelectionChangedListener(selectionChangedListener);

        //create context menu
        menuService.registerContextMenu(viewer.getControl(), "eu.etaxonomy.taxeditor.editor.popupmenu.descriptiveDataSetNavigator"); //$NON-NLS-1$

        init();
    }

    public void init(){
        if (CdmStore.isActive()){
            if(conversation == null) {
                conversation = CdmStore.createConversation();
            }
            if(cdmEntitySession == null){
                cdmEntitySession = CdmStore.getCurrentSessionManager().newSession(this, true);
            }
            descriptiveDataSets = CdmStore.getService(IDescriptiveDataSetService.class).list(DescriptiveDataSet.class, null, null, null, null);
            viewer.setInput(descriptiveDataSets);
        }
    }

    public void addDescriptiveDataSet(DescriptiveDataSet descriptiveDataSetSet){
        descriptiveDataSets.add(descriptiveDataSetSet);
        refresh();
    }

    public void removeDescriptiveDataSet(DescriptiveDataSet descriptiveDataSetSet){
        descriptiveDataSets.remove(descriptiveDataSetSet);
        refresh();
    }

    public void refresh(){
        refresh(false);
    }

    public void refresh(boolean reloadDescriptiveDataSets){
        if(reloadDescriptiveDataSets){
            cdmEntitySession.dispose();
            cdmEntitySession = CdmStore.getCurrentSessionManager().newSession(this, true);
            IDescriptiveDataSetService service = CdmStore.getService(IDescriptiveDataSetService.class);
            descriptiveDataSets = service.list(DescriptiveDataSet.class, null, null, null, null);
        }
        viewer.setInput(descriptiveDataSets);
        viewer.refresh();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void update(CdmDataChangeMap arg0) {
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public ICdmEntitySession getCdmEntitySession() {
        return cdmEntitySession;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public <T extends CdmBase> Collection<T> getRootEntities() {
        return null;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Map<Object, List<String>> getPropertyPathsMap() {
        return null;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public ConversationHolder getConversationHolder() {
        return conversation;
    }

    /** {@inheritDoc} */
    @PreDestroy
    public void dispose() {
        if (conversation != null) {
            conversation.close();
            conversation = null;
        }
        if(cdmEntitySession != null) {
            cdmEntitySession.dispose();
            cdmEntitySession = null;
        }
    }

    /** {@inheritDoc} */
    @Focus
    public void setFocus() {
        if (getConversationHolder() != null) {
            getConversationHolder().bind();
        }
        if(cdmEntitySession != null) {
            cdmEntitySession.bind();
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void contextAboutToStop(IMemento memento, IProgressMonitor monitor) {
        // TODO Auto-generated method stub

    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void contextStop(IMemento memento, IProgressMonitor monitor) {
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void contextStart(IMemento memento, IProgressMonitor monitor) {
        if(viewer!=null && viewer.getControl()!=null && !viewer.getControl().isDisposed()){
            init();
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void contextRefresh(IProgressMonitor monitor) {
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void workbenchShutdown(IMemento memento, IProgressMonitor monitor) {
    }

}
