/**
 * Copyright (C) 2018 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.editor.descriptiveDataSet.matrix;

import java.util.HashSet;
import java.util.Set;

import org.apache.commons.lang.StringUtils;
import org.eclipse.nebula.widgets.nattable.config.IConfigRegistry;
import org.eclipse.nebula.widgets.nattable.data.ListDataProvider;
import org.eclipse.nebula.widgets.nattable.style.DisplayMode;
import org.eclipse.nebula.widgets.nattable.summaryrow.DefaultSummaryRowConfiguration;
import org.eclipse.nebula.widgets.nattable.summaryrow.ISummaryProvider;
import org.eclipse.nebula.widgets.nattable.summaryrow.SummaryRowConfigAttributes;
import org.eclipse.nebula.widgets.nattable.summaryrow.SummaryRowLayer;

import eu.etaxonomy.cdm.api.service.dto.TaxonRowWrapperDTO;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.description.CategoricalData;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.QuantitativeData;

/**
 * Configuration for the aggregation algorithm for the columns
 * of the character matrix
 * @author pplitzner
 * @since Jan 4, 2018
 *
 */
public class AggregationConfiguration extends DefaultSummaryRowConfiguration {

    private CharacterMatrix matrix;

    public AggregationConfiguration(CharacterMatrix matrix) {
        this.matrix = matrix;
    }

    @Override
    public void addSummaryProviderConfig(IConfigRegistry configRegistry) {
        for(int i=0;i<matrix.getFeatures().size();i++){
            //no summary for the supplemental columns
            if(i<CharacterMatrix.LEADING_COLUMN_COUNT){
                int index = i;
                configRegistry.registerConfigAttribute(
                        SummaryRowConfigAttributes.SUMMARY_PROVIDER,
                        columnIndex -> "",
                        DisplayMode.NORMAL,
                        SummaryRowLayer.DEFAULT_SUMMARY_COLUMN_CONFIG_LABEL_PREFIX+index);
            }
            //register aggregation configuration for each feature
            Feature feature = matrix.getFeatures().get(i);
            if(feature.isSupportsQuantitativeData()){
                configRegistry.registerConfigAttribute(
                        SummaryRowConfigAttributes.SUMMARY_PROVIDER,
                        new QuantitativeSummaryProvider(),
                        DisplayMode.NORMAL,
                        SummaryRowLayer.DEFAULT_SUMMARY_COLUMN_CONFIG_LABEL_PREFIX+MatrixUtility.getProperty(feature));
            }
            if(feature.isSupportsCategoricalData()){
                configRegistry.registerConfigAttribute(
                        SummaryRowConfigAttributes.SUMMARY_PROVIDER,
                        new CategoricalSummaryProvider(),
                        DisplayMode.NORMAL,
                        SummaryRowLayer.DEFAULT_SUMMARY_COLUMN_CONFIG_LABEL_PREFIX+MatrixUtility.getProperty(feature));
            }
        }
    }

    private class QuantitativeSummaryProvider implements ISummaryProvider {
        @Override
        public Object summarize(int columnIndex) {
            Float minTotal = null;
            Float maxTotal = null;
            ListDataProvider<Object> bodyDataProvider = AggregationConfiguration.this.matrix.getBodyDataProvider();
            int rowCount = bodyDataProvider.getRowCount();

            for (int rowIndex = 0; rowIndex < rowCount; rowIndex++) {
                Object rowObject = bodyDataProvider.getRowObject(rowIndex);
                if(rowObject instanceof TaxonRowWrapperDTO){
                    // do not aggregate taxon descriptions
                    continue;
                }
                Object dataValue = bodyDataProvider.getDataValue(columnIndex,
                        rowIndex);
                if(HibernateProxyHelper.isInstanceOf(dataValue, QuantitativeData.class)){
                    QuantitativeData quantitativeData = HibernateProxyHelper.deproxy(dataValue, QuantitativeData.class);
                    Float min = quantitativeData.getMin();
                    Float max = quantitativeData.getMax();
                    if(min!=null && (minTotal==null || min<minTotal)){
                        minTotal = min;
                    }
                    if(max!=null && (maxTotal==null || max>maxTotal)){
                        maxTotal = max;
                    }
                }
            }
            String summaryString = MatrixUtility.getQuantitativeLabel(minTotal, null, maxTotal);
            return summaryString;
        }

    }

    private class CategoricalSummaryProvider implements ISummaryProvider {
        @Override
        public Object summarize(int columnIndex) {
            int rowCount = AggregationConfiguration.this.matrix.getBodyDataProvider().getRowCount();
            Set states = new HashSet<>();
            for (int rowIndex = 0; rowIndex < rowCount; rowIndex++) {
                Object dataValue = AggregationConfiguration.this.matrix.getBodyDataProvider().getDataValue(columnIndex,
                        rowIndex);
                if(dataValue instanceof CategoricalData){
                    states.addAll(((CategoricalData) dataValue).getStatesOnly());
                }
            }
            return StringUtils.join(states, ", "); //$NON-NLS-1$
        }
    }
}
