/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.descriptiveDataSet.matrix;

import java.util.List;

import org.eclipse.jface.window.ToolTip;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.swtchart.Chart;
import org.swtchart.IAxis;
import org.swtchart.IBarSeries;
import org.swtchart.ISeries.SeriesType;
import org.swtchart.ISeriesLabel;

import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.State;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;

/**
 * @author pplitzner
 * @since Jul 6, 2018
 *
 */
public class CategoricalChartTooltip extends ToolTip {

    static final String TOOLTIP_NOT_AVAILABLE = Messages.CategoricalChartTooltip_TOOLTIP_NOT_AVAILABLE;

    private CharacterMatrix matrix;

    public CategoricalChartTooltip(CharacterMatrix matrix) {
        super(matrix.getNatTable(), ToolTip.NO_RECREATE, false);
        this.matrix = matrix;
    }

    @Override
    protected Object getToolTipArea(Event event) {
        int col = matrix.getNatTable().getColumnPositionByX(event.x);
        int row = matrix.getNatTable().getRowPositionByY(event.y);
        return new Point(col, row);
    }

    @Override
    protected Composite createToolTipContentArea(Event event,
            Composite parent) {

        int colPos = matrix.getNatTable().getColumnPositionByX(event.x);
        int colIndex = matrix.getNatTable().getColumnIndexByPosition(colPos);
        Feature feature = matrix.getIndexToCharacterMap().get(colIndex);
        CategoricalDataHistogram histogram = matrix.getFeatureToHistogramMap().get(feature);

        Composite tooltip;
        if(histogram!=null && !histogram.getStates().isEmpty()){
            parent.setLayout(new GridLayout());
            tooltip = createTooltip(event, parent);
        }
        else{
            tooltip = new Composite(parent, SWT.NONE);
            tooltip.setLayout(new FillLayout());
            Label label = new Label(tooltip, SWT.NONE);
            label.setText(TOOLTIP_NOT_AVAILABLE);
        }
        return tooltip;
    }

    private Composite createTooltip(Event event, Composite parent) {
        Chart chart = new Chart(parent, SWT.NONE);
        chart.setLayoutData(new GridData(600, 300));

        //get histogram for column
        int colPos = matrix.getNatTable().getColumnPositionByX(event.x);
        int colIndex = matrix.getNatTable().getColumnIndexByPosition(colPos);
        Feature feature = matrix.getIndexToCharacterMap().get(colIndex);
        CategoricalDataHistogram histogram = matrix.getFeatureToHistogramMap().get(feature);

        // set chart title
        chart.getTitle().setText(feature.getLabel());
        IAxis xAxis = chart.getAxisSet().getXAxis(0);

        //initalize x and y axis values
        List<State> statesList = histogram.getStates();
        String[] states = new String[statesList.size()];
        double[] counts = new double[statesList.size()];

        int index = 0;
        for (State state : statesList) {
            states[index] = state.getLabel();
            counts[index] = histogram.getCount(state);
            index++;
        }
        xAxis.setCategorySeries(states);
        xAxis.enableCategory(true);
        xAxis.getTitle().setVisible(false);
        xAxis.getTick().setTickLabelAngle(25);

        IAxis yAxis = chart.getAxisSet().getYAxis(0);
        yAxis.getTitle().setVisible(false);

        chart.getLegend().setVisible(false);

        IBarSeries barSeries = (IBarSeries) chart.getSeriesSet()
            .createSeries(SeriesType.BAR, "state counts"); //$NON-NLS-1$
        ISeriesLabel seriesLabel = barSeries.getLabel();
        seriesLabel.setVisible(true);
        barSeries.setYSeries(counts);

        chart.getAxisSet().adjustRange();

        return chart;
    }

    @Override
    protected boolean shouldCreateToolTip(Event event) {
        //Differentiation between "index" and "position" is important here
        //"position" is the current visible index
        //"index" refers to the underlying data model
        int colPos = matrix.getNatTable().getColumnPositionByX(event.x);
        int colIndex = matrix.getNatTable().getColumnIndexByPosition(colPos);
        int rowPos = matrix.getNatTable().getRowPositionByY(event.y);
        Feature feature = matrix.getIndexToCharacterMap().get(colIndex);

        if(matrix.isShowTooltips()
                && rowPos==1
                && colIndex>=CharacterMatrix.LEADING_COLUMN_COUNT
                && feature!=null
                && feature.isSupportsCategoricalData()) {
            return true;
        }
        return false;
    }

}
