/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.descriptiveDataSet.matrix;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Collection;
import java.util.Properties;
import java.util.function.Consumer;

import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.nebula.widgets.nattable.export.command.ExportCommand;
import org.eclipse.nebula.widgets.nattable.persistence.PersistenceHelper;
import org.eclipse.nebula.widgets.nattable.persistence.command.DisplayPersistenceDialogCommand;
import org.eclipse.nebula.widgets.nattable.persistence.command.DisplayPersistenceDialogCommandHandler;
import org.eclipse.nebula.widgets.nattable.persistence.command.IStateChangedListener;
import org.eclipse.nebula.widgets.nattable.persistence.command.StateChangeEvent;
import org.eclipse.nebula.widgets.nattable.persistence.gui.PersistenceDialog;
import org.eclipse.nebula.widgets.nattable.tree.command.TreeCollapseAllCommand;
import org.eclipse.nebula.widgets.nattable.tree.command.TreeExpandAllCommand;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.ImageResources;
import eu.etaxonomy.taxeditor.model.MessagingUtils;

/**
 * @author pplitzner
 * @since Jul 9, 2018
 *
 */
public class CharacterMatrixToolbar extends Composite {

    private CharacterMatrix matrix;
    private Label wsLabel;
    private DisplayPersistenceDialogCommandHandler displayPersistenceDialogCommandHandler;
    private Properties natTableState;

    public CharacterMatrixToolbar(CharacterMatrix matrix, int style) {
        super(matrix, style);
        this.matrix = matrix;

        init();
    }

    private void init() {
        setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
        setLayout(new GridLayout(10, false));

        wsLabel = new Label(this, SWT.NONE);

        Button btnToggleTooltips = new Button(this, SWT.TOGGLE);
        Button btnToggleTree = new Button(this, SWT.PUSH);
        Button btnToggleFlat = new Button(this, SWT.PUSH);
        Button btnCollapseAll = new Button(this, SWT.PUSH);
        Button btnExpandAll = new Button(this, SWT.PUSH);
        Button btnFreezeSuppInfo = new Button(this, SWT.TOGGLE);
        ComboViewer comboStates = new ComboViewer(this, SWT.DROP_DOWN);
        Button btnManageState = new Button(this, SWT.PUSH);
        Button btnExcelExport = new Button(this, SWT.PUSH);

        /**
         * Toggle tooltips button
         */
        initButton(
                btnToggleTooltips,
                ImageResources.getImage(ImageResources.LIGHT_BULB),
                "Show tooltips",
                null,
                true,
                true,
                (e)->matrix.toogleIsShowTooltips()
                );
        /**
         * Toogle tree button
         */
        initButton(
                btnToggleTree,
                ImageResources.getImage(ImageResources.HIERARCHICAL),
                Messages.CharacterMatrix_SHOW_HIERARCHY,
                null,
                false,
                true,
                (e)->matrix.toggleTreeFlat(true, btnToggleFlat, btnToggleTree, btnCollapseAll, btnExpandAll, btnFreezeSuppInfo)
                );

        /**
         * Toogle flat button
         */
        initButton(
                btnToggleFlat,
                ImageResources.getImage(ImageResources.FLAT),
                Messages.CharacterMatrix_SHOW_FLAT_LIST,
                null,
                true,
                false,
                (e)->matrix.toggleTreeFlat(false, btnToggleFlat, btnToggleTree, btnCollapseAll, btnExpandAll, btnFreezeSuppInfo)
                );

        /**
         *
         * Collapse button
         */
        initButton(
                btnCollapseAll,
                ImageResources.getImage(ImageResources.COLLAPSE_ALL),
                Messages.CharacterMatrix_COLLAPSE,
                null,
                true,
                false,
                (e)->matrix.getNatTable().doCommand(new TreeCollapseAllCommand())
                );

        /**
         * Expand button
         */
        initButton(
                btnExpandAll,
                ImageResources.getImage(ImageResources.EXPAND_ALL),
                Messages.CharacterMatrix_EXPAND,
                null,
                true,
                false,
                (e)->matrix.getNatTable().doCommand(new TreeExpandAllCommand())
                );

        /**
         * Freeze supplemental info button
         */
        initButton(
                btnFreezeSuppInfo,
                ImageResources.getImage(ImageResources.LOCK_ICON),
                Messages.CharacterMatrix_LOCK_COLUMNS,
                null,
                true,
                true,
                (e)->{
                    boolean isSelected = btnFreezeSuppInfo.getSelection();
                    matrix.freezeSupplementalColumns(isSelected);
                    btnFreezeSuppInfo.setImage(isSelected?
                            ImageResources.getImage(ImageResources.LOCK_ICON):
                                ImageResources.getImage(ImageResources.LOCK_OPEN_ICON));
                }
                );

        /**
         * Table state persistence
         */
        natTableState = new Properties();
        //load persisted state
        File statePropertiesFile = matrix.getStatePropertiesFile();
        FileInputStream inputStream;
        try {
            inputStream = new FileInputStream(statePropertiesFile);
            natTableState.load(inputStream);
        } catch (IOException e) {
            MessagingUtils.info("No initial state properties file found for character matrix"); //$NON-NLS-1$
        }

        // create a combobox for showing the available view states
        Collection<String> availableStates = PersistenceHelper.getAvailableStates(natTableState);
        comboStates.setLabelProvider(new LabelProvider(){
            @Override
            public String getText(Object element) {
                if(element instanceof String && ((String) element).isEmpty()){
                    return Messages.CharacterMatrix_DEFAULT;
                }
                return super.getText(element);
            }
        });
        comboStates.setContentProvider(new ArrayContentProvider());
        comboStates.addSelectionChangedListener(e->
        {
            int index = comboStates.getCombo().getSelectionIndex();
            if (index >= 0) {
                String selected = comboStates.getCombo().getItem(index);
                // load the state
                matrix.getNatTable().loadState(selected, natTableState);
                natTableState.setProperty(PersistenceDialog.ACTIVE_VIEW_CONFIGURATION_KEY, selected);
            }
        });
        comboStates.setInput(availableStates);
        if(comboStates.getCombo().getItemCount()>0){
            comboStates.getCombo().select(0);
        }

        displayPersistenceDialogCommandHandler = new DisplayPersistenceDialogCommandHandler(natTableState, matrix.getNatTable());
        // add listener to update the combo on view state management changes
        displayPersistenceDialogCommandHandler.addStateChangeListener(new IStateChangedListener() {
            @Override
            public void handleStateChange(StateChangeEvent event) {
                comboStates.setInput(PersistenceHelper.getAvailableStates(natTableState));
                matrix.selectStateItem(comboStates, event.getViewConfigName());
            }
        });

        // add button to show dialog
        btnManageState.setImage(ImageResources.getImage(ImageResources.SETTINGS));
        btnManageState.setToolTipText(Messages.CharacterMatrix_VIEW_CONFIG);
        btnManageState.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                matrix.getNatTable().doCommand(new DisplayPersistenceDialogCommand(matrix.getNatTable()));
                Object activeConfig = natTableState.get(PersistenceDialog.ACTIVE_VIEW_CONFIGURATION_KEY);
                if(activeConfig!=null){
                    matrix.selectStateItem(comboStates, activeConfig.toString());
                }
            }
        });

        /**
         * excel export
         */
        btnExcelExport.setToolTipText(Messages.CharacterMatrix_EXPORT);
        btnExcelExport.setImage(ImageResources.getImage(ImageResources.EXPORT));
        btnExcelExport.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                matrix.getNatTable().doCommand(
                        new ExportCommand(
                                matrix.getNatTable().getConfigRegistry(),
                                matrix.getNatTable().getShell()));
            }
        });

    }

    private void initButton(Button button, Image image, String tooltipText,
            String label, boolean enabled, boolean selected, Consumer<SelectionEvent> widgetSelected){
        if(image!=null){
            button.setImage(image);
        }
        if(label!=null){
            button.setText(label);
        }
        if(tooltipText!=null){
            button.setToolTipText(tooltipText);
        }
        button.setSelection(selected);
        button.setEnabled(enabled);
        button.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                widgetSelected.accept(e);
            }
        });
    }

    public Label getWsLabel() {
        return wsLabel;
    }

    public Properties getNatTableState() {
        return natTableState;
    }

    public DisplayPersistenceDialogCommandHandler getDisplayPersistenceDialogCommandHandler() {
        return displayPersistenceDialogCommandHandler;
    }
}
