/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.descriptiveDataSet.matrix;

import java.util.Set;
import java.util.stream.Collectors;

import eu.etaxonomy.cdm.api.service.dto.TaxonRowWrapperDTO;
import eu.etaxonomy.cdm.model.common.MarkerType;
import eu.etaxonomy.cdm.model.description.DescriptiveDataSet;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.QuantitativeData;
import eu.etaxonomy.cdm.model.description.StatisticalMeasure;
import eu.etaxonomy.cdm.model.taxon.Classification;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;

/**
 * Utility class for the character matrix editor
 * @author pplitzner
 * @since Jan 4, 2018
 *
 */
public class MatrixUtility {

    /**
     * Returns the column property string for the given {@link Feature}
     * @param feature
     * @return
     */
    public static String getProperty(Feature feature){
        return feature.getLabel();
    }

    /**
     * Returns a string representation for the given {@link QuantitativeData}
     * @param data the quantitative data
     * @return a string representation of the data
     */
    public static String getQuantitativeLabel(QuantitativeData data) {
        String label = "";
        Float min = data.getMin();
        Float max = data.getMax();
        if(min!=null||max!=null){
            label += "["+(min!=null?min.toString():"?")+"-"+(max!=null?max.toString():"?")+"] ";
        }
        label += data.getStatisticalValues().stream().
        filter(value->value.getType().equals(StatisticalMeasure.EXACT_VALUE()))
        .map(exact->Float.toString(exact.getValue()))
        .collect(Collectors.joining(", "));

        return label;
    }

    /**
     * Returns a string representation for the given min/max values
     * @param minTotal the min value
     * @param maxTotal the max value
     * @return a string representation of the data
     */
    public static String getQuantitativeLabel(Float min, Float exact, Float max) {
        return (min==null?"":"("+min.toString()+")") //$NON-NLS-1$
                +(exact==null?"":exact.toString()) //$NON-NLS-1$
                +(max==null?"":"("+max.toString()+")"); //$NON-NLS-1$
    }

    public static Classification getClassificationForDescriptiveDataSet(DescriptiveDataSet descriptiveDataSet){
        Set<TaxonNode> taxonSubtreeFilter = descriptiveDataSet.getTaxonSubtreeFilter();
        if(taxonSubtreeFilter!=null && !taxonSubtreeFilter.isEmpty()){
            return taxonSubtreeFilter.iterator().next().getClassification();
        }
        return null;
    }

    public static boolean isDefaultTaxonDescription(TaxonRowWrapperDTO taxonRowWrapperDTO){
        return hasMarker(taxonRowWrapperDTO, MarkerType.TO_BE_CHECKED());
    }

    public static boolean isAggregatedTaxonDescription(TaxonRowWrapperDTO taxonRowWrapperDTO){
        return hasMarker(taxonRowWrapperDTO, MarkerType.COMPUTED());
    }

    public static boolean isLiteratureTaxonDescription(TaxonRowWrapperDTO taxonRowWrapperDTO){
        return hasMarker(taxonRowWrapperDTO, MarkerType.IN_BIBLIOGRAPHY());
    }

    private static boolean hasMarker(TaxonRowWrapperDTO taxonRowWrapperDTO, MarkerType markerType){
        return taxonRowWrapperDTO.getDescription().getMarkers().stream()
        .anyMatch(marker->marker.getMarkerType().equals(markerType));
    }
}

