/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.descriptiveDataSet.matrix;

import java.util.List;

import org.apache.commons.math3.stat.descriptive.DescriptiveStatistics;
import org.eclipse.jface.window.DefaultToolTip;
import org.eclipse.jface.window.ToolTip;
import org.eclipse.nebula.widgets.nattable.util.GUIHelper;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.swtchart.Chart;
import org.swtchart.IAxis;
import org.swtchart.ILineSeries;
import org.swtchart.ILineSeries.PlotSymbolType;
import org.swtchart.ISeries.SeriesType;
import org.swtchart.LineStyle;

import eu.etaxonomy.cdm.model.description.Feature;

/**
 * @author pplitzner
 * @since Jul 6, 2018
 *
 */
public class QuantitativeChartTooltip extends DefaultToolTip {

    private CharacterMatrix matrix;

    public QuantitativeChartTooltip(CharacterMatrix matrix) {
        super(matrix.getNatTable(), ToolTip.NO_RECREATE, false);
        this.matrix = matrix;
    }

    @Override
    protected Object getToolTipArea(Event event) {
        int col = matrix.getNatTable().getColumnPositionByX(event.x);
        int row = matrix.getNatTable().getRowPositionByY(event.y);
        return new Point(col, row);
    }

    @Override
    protected Composite createToolTipContentArea(Event event,
            Composite parent) {

        int colPos = matrix.getNatTable().getColumnPositionByX(event.x);
        int colIndex = matrix.getNatTable().getColumnIndexByPosition(colPos);
        Feature feature = matrix.getIndexToCharacterMap().get(colIndex);
        QuantitativeDataStatistics dataStatistics = matrix.getFeatureToQuantDataStatisticsMap().get(feature);

        Composite tooltip;
        if(dataStatistics!=null){
            parent.setLayout(new GridLayout());
            tooltip = createTooltip(event, parent);
        }
        else{
            tooltip = new Composite(parent, SWT.NONE);
            tooltip.setLayout(new FillLayout());
            Label label = new Label(tooltip, SWT.NONE);
            label.setText(CategoricalChartTooltip.TOOLTIP_NOT_AVAILABLE);
        }
        return tooltip;

    }

    private Composite createTooltip(Event event, Composite parent){
        Chart chart = new Chart(parent, SWT.NONE);
        chart.setLayoutData(new GridData(600, 300));

        //get statistics for column
        int colPos = matrix.getNatTable().getColumnPositionByX(event.x);
        int colIndex = matrix.getNatTable().getColumnIndexByPosition(colPos);
        Feature feature = matrix.getIndexToCharacterMap().get(colIndex);
        QuantitativeDataStatistics quantitativeDataStatistics = matrix.getFeatureToQuantDataStatisticsMap().get(feature);
        List<DescriptiveStatistics> statistics = quantitativeDataStatistics.getStatistics();

        //prepare series
        double[] exactMean = statistics.stream().mapToDouble(statistic->statistic.getMean()).toArray();
        double[] mins = statistics.stream().mapToDouble(statistic->statistic.getMin()).toArray();
        double[] maxs = statistics.stream().mapToDouble(statistic->statistic.getMax()).toArray();

        // set chart title
        chart.getTitle().setText(feature.getLabel());

        // create exact/mean series
        ILineSeries meanExactSeries = (ILineSeries) chart.getSeriesSet()
                .createSeries(SeriesType.LINE, "exact/mean");
        meanExactSeries.setYSeries(exactMean);
        meanExactSeries.setLineStyle(LineStyle.NONE);
        meanExactSeries.setSymbolColor(GUIHelper.COLOR_BLUE);

        // create min series
        ILineSeries minSeries = (ILineSeries) chart.getSeriesSet()
                .createSeries(SeriesType.LINE, "min");
        minSeries.setYSeries(mins);
        minSeries.setLineStyle(LineStyle.NONE);
        minSeries.setSymbolType(PlotSymbolType.INVERTED_TRIANGLE);
        minSeries.setSymbolColor(GUIHelper.COLOR_GREEN);

        // create max series
        ILineSeries maxSeries = (ILineSeries) chart.getSeriesSet()
                .createSeries(SeriesType.LINE, "max");
        maxSeries.setYSeries(maxs);
        maxSeries.setLineStyle(LineStyle.NONE);
        maxSeries.setSymbolType(PlotSymbolType.TRIANGLE);
        maxSeries.setSymbolColor(GUIHelper.COLOR_RED);

        IAxis yAxis = chart.getAxisSet().getYAxis(0);
        yAxis.getTitle().setVisible(false);
        IAxis xAxis = chart.getAxisSet().getXAxis(0);
        xAxis.getTitle().setVisible(false);

        chart.getAxisSet().adjustRange();

        return chart;
    }

    @Override
    protected boolean shouldCreateToolTip(Event event) {
        //Differentiation between "index" and "position" is important here
        //"position" is the current visible index
        //"index" refers to the underlying data model
        int colPos = matrix.getNatTable().getColumnPositionByX(event.x);
        int colIndex = matrix.getNatTable().getColumnIndexByPosition(colPos);
        int rowPos = matrix.getNatTable().getRowPositionByY(event.y);
        Feature feature = matrix.getIndexToCharacterMap().get(colIndex);

        if(matrix.isShowTooltips()
                && rowPos==1
                && feature!=null
                && feature.isSupportsQuantitativeData()
                && colIndex>=CharacterMatrix.LEADING_COLUMN_COUNT) {
            return true;
        }
        return false;
    }

}
