/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.descriptiveDataSet.matrix;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.cdm.api.service.IDescriptiveDataSetService;
import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.persistence.dto.SpecimenNodeWrapper;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.model.ImageResources;
import eu.etaxonomy.taxeditor.preference.Resources;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.combo.taxon.TaxonNodeCombo;

/**
 * Dialog to choose specimens for the character matrix.<br>
 * Filters specimens that have already been added.
 * @author pplitzner
 * @since Jan 3, 2018
 *
 */
public class SpecimenSelectionDialog extends Dialog {

    private static String TEXT_FILTER_DEFAULT = "Enter text filter";

    private TableViewer list;

    private Collection<SpecimenNodeWrapper> selectedSpecimens = new ArrayList<>();
    private CharacterMatrix matrix;
    private Text txtTextFilter;

    private TaxonNodeCombo comboTaxon;

    protected SpecimenSelectionDialog(Shell parentShell, CharacterMatrix matrix) {
        super(parentShell);
        this.matrix = matrix;
    }

    @Override
    protected Control createDialogArea(Composite parent) {
        Composite composite = (Composite) super.createDialogArea(parent);
        GridLayout gl_composite = new GridLayout();
        composite.setLayout(gl_composite);

        Composite composite_1 = new Composite(composite, SWT.NONE);
        composite_1.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false, 1, 1));
        composite_1.setLayout(new GridLayout(4, false));

        Label lblNewLabel = new Label(composite_1, SWT.NONE);
        lblNewLabel.setText("Taxon");

        comboTaxon = new TaxonNodeCombo(composite_1, SWT.NONE);
        comboTaxon.setInput(matrix.getDescriptiveDataSet().getTaxonSubtreeFilter());
        comboTaxon.addSelectionChangedListener(e->applyFilter());

        txtTextFilter = new Text(composite_1, SWT.BORDER);
        txtTextFilter.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        txtTextFilter.setText(TEXT_FILTER_DEFAULT);
        txtTextFilter.addFocusListener(new FocusListener() {

            @Override
            public void focusGained(FocusEvent e) {
                txtTextFilter.setForeground(AbstractUtility.getColor(Resources.SEARCH_VIEW_FOCUS));
                if (TEXT_FILTER_DEFAULT.equals(txtTextFilter.getText())) {
                    txtTextFilter.setText("");
                }
            }

            @Override
            public void focusLost(FocusEvent e) {
                if (CdmUtils.isBlank(txtTextFilter.getText())){
                    txtTextFilter.setForeground(AbstractUtility.getColor(Resources.SEARCH_VIEW_FOREGROUND));
                    txtTextFilter.setText(TEXT_FILTER_DEFAULT);
                }
            }
        });
        txtTextFilter.addModifyListener(e->applyFilter());

        Button btnRefreshButton = new Button(composite_1, SWT.NONE);
        btnRefreshButton.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        btnRefreshButton.setToolTipText(Messages.SpecimenSelectionDialog_REFRESH);
        btnRefreshButton.setImage(ImageResources.getImage(ImageResources.REFRESH));
        btnRefreshButton.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                loadSpecimens();
                list.setInput(matrix.getSpecimenCache());
            }
        });

        ScrolledComposite scrolledComposite = new ScrolledComposite(composite, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
        scrolledComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
        scrolledComposite.setExpandHorizontal(true);
        scrolledComposite.setExpandVertical(true);
        //table
        list = new TableViewer(scrolledComposite, SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION | SWT.MULTI);
        Table table = list.getTable();

        //table columns
        TableViewerColumn columnTaxon = new TableViewerColumn(list, SWT.NONE);
        columnTaxon.getColumn().setText("Taxon");
        columnTaxon.getColumn().setWidth(200);
        columnTaxon.getColumn().setResizable(true);
        columnTaxon.getColumn().setMoveable(true);
        TableViewerColumn columnSpecimen = new TableViewerColumn(list, SWT.NONE);
        columnSpecimen.getColumn().setText("Specimen");
        columnSpecimen.getColumn().setResizable(true);
        columnSpecimen.getColumn().setMoveable(true);
        columnSpecimen.getColumn().setWidth(400);
        table.setHeaderVisible(true);
        table.setLinesVisible(true);
        scrolledComposite.setContent(table);
        scrolledComposite.setMinSize(table.computeSize(SWT.DEFAULT, SWT.DEFAULT));
        list.setContentProvider(new ArrayContentProvider());
        list.setLabelProvider(new SpecimenListLabelProvider());

        //gather specimens
        if(matrix.getSpecimenCache()==null){
            loadSpecimens();
        }

        list.setInput(matrix.getSpecimenCache());

        columnSpecimen.getColumn().pack();

        return composite;
    }

    private void applyFilter(){
        Collection<SpecimenNodeWrapper> specimenCache = matrix.getSpecimenCache();
        List<SpecimenNodeWrapper> result = new ArrayList<>();
        if(comboTaxon.getSelection()!=null){
            result.addAll(specimenCache.stream()
                    .filter(wrapper->wrapper.getTaxonNode().equals(comboTaxon.getSelection()))
                    .collect(Collectors.toList()));
        }
        if(!txtTextFilter.getForeground().equals(AbstractUtility.getColor(Resources.SEARCH_VIEW_FOREGROUND))
                &&!CdmUtils.isBlank(txtTextFilter.getText())){
            result.addAll(specimenCache.stream()
                    .filter(wrapper->wrapper.getUuidAndTitleCache().getTitleCache().toLowerCase().contains(txtTextFilter.getText().toLowerCase()))
                    .collect(Collectors.toList()));
        }
        list.setInput(result);
    }

    private void loadSpecimens(){
        Collection<SpecimenNodeWrapper> specimenCache = CdmStore.getService(IDescriptiveDataSetService.class).loadSpecimens(matrix.getDescriptiveDataSet());
        matrix.setSpecimenCache(specimenCache);
    }

    @Override
    protected void configureShell(Shell newShell) {
        super.configureShell(newShell);
        newShell.setText(Messages.SpecimenSelectionDialog_SELECT_SPECIMENS);
        newShell.setMinimumSize(500, 600);
        newShell.setSize(800, 600);
    }

    @Override
    protected void okPressed() {
        selectedSpecimens.clear();
        selectedSpecimens = ((IStructuredSelection)list.getSelection()).toList();
        super.okPressed();
    }

    @Override
    protected boolean isResizable() {
        return true;
    }

    public Collection<SpecimenNodeWrapper> getSpecimen(){
        return selectedSpecimens;
    }

    private class SpecimenListLabelProvider extends LabelProvider implements ITableLabelProvider{

        @Override
        public Image getColumnImage(Object element, int columnIndex) {
            return null;
        }

        @Override
        public String getColumnText(Object element, int columnIndex) {
            if(element instanceof SpecimenNodeWrapper){
                SpecimenNodeWrapper wrapper = (SpecimenNodeWrapper)element;
                switch (columnIndex) {
                case 0:
                    return wrapper.getTaxonNode().getTaxon().getName().getTitleCache();
                case 1:
                    return wrapper.getUuidAndTitleCache().getTitleCache();
                default:
                    break;
                }
            }
            return "";
        }
    }
}
