/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.editor.name.e4.container;

import java.util.Iterator;

import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.Assert;
import org.eclipse.jface.text.Document;
import org.eclipse.jface.text.IUndoManager;
import org.eclipse.jface.text.IUndoManagerExtension;
import org.eclipse.jface.text.Position;
import org.eclipse.jface.text.TextViewerUndoManager;
import org.eclipse.jface.text.source.Annotation;
import org.eclipse.jface.text.source.AnnotationModel;
import org.eclipse.jface.text.source.AnnotationPainter;
import org.eclipse.jface.text.source.IAnnotationAccess;
import org.eclipse.jface.text.source.IVerticalRuler;
import org.eclipse.jface.text.source.SourceViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Layout;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.forms.widgets.TableWrapData;
import org.eclipse.ui.forms.widgets.TableWrapLayout;
import org.eclipse.ui.operations.OperationHistoryActionHandler;
import org.eclipse.ui.operations.RedoActionHandler;
import org.eclipse.ui.operations.UndoActionHandler;
import org.eclipse.ui.texteditor.DefaultMarkerAnnotationAccess;
import org.eclipse.ui.texteditor.IWorkbenchActionDefinitionIds;

import eu.etaxonomy.taxeditor.editor.ViewerConfiguration;
import eu.etaxonomy.taxeditor.editor.name.container.EditorAnnotation;
import eu.etaxonomy.taxeditor.editor.name.container.EditorAnnotation.EditorAnnotationType;
import eu.etaxonomy.taxeditor.editor.name.container.LineWrapSquigglesStrategy;
import eu.etaxonomy.taxeditor.editor.name.container.RulerWithIcon;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.preference.Resources;

/**
 *
 * @author pplitzner
 * @since Aug 25, 2017
 *
 */
public class NameViewerE4 extends SourceViewer {

	public static final int RULER_WIDTH = 16;

	private final IVerticalRuler ruler;
	private final AnnotationModel annotationModel;

	public NameViewerE4(Composite parent) {
		super(parent, new RulerWithIcon(RULER_WIDTH), SWT.WRAP | SWT.MULTI | SWT.RESIZE);

		this.ruler = getVerticalRuler();

		setBackground(AbstractUtility.getColor(Resources.COLOR_COMPOSITE_BACKGROUND));

		Canvas canvas = (Canvas) getControl();
		TableWrapLayout layout = (TableWrapLayout) canvas.getLayout();
		layout.topMargin = 0;
		layout.rightMargin = 0;
		layout.bottomMargin = 0;
		layout.leftMargin = 0;
		layout.verticalSpacing = 10;
		layout.horizontalSpacing = 0;

		canvas.setLayout(layout);

		// Lay out the viewer's widgets
		TableWrapData twdata = new TableWrapData(TableWrapData.FILL_GRAB);
		getControl().setLayoutData(twdata);

		twdata = new TableWrapData(TableWrapData.FILL_GRAB);
		getTextWidget().setLayoutData(twdata);
		getTextWidget().setLineSpacing(5);

		// Default implementation adds ruler after the text widget, then
		//	uses a custom layout to display it before the text widget
		ruler.getControl().moveAbove(getTextWidget());

		annotationModel = new AnnotationModel();
		this.setDocument(new Document(""), annotationModel); //$NON-NLS-1$

		createAnnotationPainter();

		this.configure(new ViewerConfiguration());

	}

	public void setBackground(Color color) {

		// Set background color of ruler
		ruler.getControl().setBackground(color);

		// Set background color of text widget
		getTextWidget().setBackground(color);

		// Set background color of strip between ruler and textWidget
		getTextWidget().getParent().setBackground(color);
	}

	public Control getRulerControl() {
		return ruler.getControl();
	}

	public void setIcon(Image icon) {
		if (ruler instanceof RulerWithIcon) {
			((RulerWithIcon) ruler).setIcon(icon);
		} else {
			throw new IllegalStateException("Viewer's IVerticalRuler is not an instance of RulerWithIcon."); //$NON-NLS-1$
		}
	}

	@Override
	protected Layout createLayout() {
		TableWrapLayout layout = new TableWrapLayout();
		layout.numColumns = 2;
		layout.verticalSpacing = 0;
		return layout;
	}

	private void createAnnotationPainter(){
		// Annotations section
		IAnnotationAccess fAnnotationAccess = new DefaultMarkerAnnotationAccess();

		// To paint the annotations
		AnnotationPainter annotationPainter = new AnnotationPainter(this, fAnnotationAccess);

		// Default SquigglesStrategy doesn't recognize line wraps
		annotationPainter.addDrawingStrategy(LineWrapSquigglesStrategy.ID, new LineWrapSquigglesStrategy());

		// Add ability to paint red squigglies
		annotationPainter.addAnnotationType(EditorAnnotationType.ERROR.name(), LineWrapSquigglesStrategy.ID);
		annotationPainter.setAnnotationTypeColor(EditorAnnotationType.ERROR.image,
				new Color(Display.getDefault(), EditorAnnotationType.ERROR.color));

		// Add ability to paint yellow squigglies
		annotationPainter.addAnnotationType(EditorAnnotationType.WARNING.name(), LineWrapSquigglesStrategy.ID);
		annotationPainter.setAnnotationTypeColor(EditorAnnotationType.WARNING.image,
				new Color(Display.getDefault(), EditorAnnotationType.WARNING.color));

		this.addPainter(annotationPainter);
	}


	public void clearAnnotations() {
		Iterator<Annotation> annotations = this.getAnnotationModel().getAnnotationIterator();
		while (annotations.hasNext()) {
			Annotation annotation = annotations.next();
			this.getAnnotationModel().removeAnnotation(annotation);
		}
	}

	public void addAnnotation(EditorAnnotation annotation){
		addAnnotation(annotation, null);
	}

	public void addAnnotation(EditorAnnotation annotation, Position position){
		if(position == null){
			position = new Position(0, 0);
		}

		this.getAnnotationModel().addAnnotation(annotation, position);
	}

	public void setText(String text) {
		if (text == null) {
			text = ""; //$NON-NLS-1$
		}
		try {
			Assert.isNotNull(text);
			// TODO figure out why getTextWidget() returns null!
			if (this.getTextWidget() == null) {
				return;
			}
			Assert.isNotNull(this.getTextWidget());
			this.getTextWidget().setText(text);
		} catch (RuntimeException e) {
			throw e;
		}
	}

	public void setMenu(Menu menu) {
		getRulerControl().setMenu(menu);
		getTextWidget().setMenu(menu);
	}

	public void setCursorToEOL() {
		getTextWidget().setCaretOffset(getTextWidget().getText().length());
		getTextWidget().setFocus();
	}

	public int getCursorPosition(){
		return getTextWidget().getCaretOffset();
	}

	public void setCursorPosition(int offset){
		try{
			getTextWidget().setCaretOffset(offset);
		}catch(IllegalArgumentException e){
			// do nothing
		}
	}

	public void createUndoSupport(IEditorSite editorSite) {

		IUndoManager undoManager = new TextViewerUndoManager(25);
		this.setUndoManager(undoManager);
		undoManager.connect(this);

		IUndoContext workbenchUndoContext = ((IUndoManagerExtension)undoManager).getUndoContext();

		OperationHistoryActionHandler undoAction = new UndoActionHandler(editorSite, workbenchUndoContext);
		undoAction.setActionDefinitionId(ActionFactory.UNDO.getId());

		// Create the redo action.
		OperationHistoryActionHandler redoAction = new RedoActionHandler(editorSite, workbenchUndoContext);
		undoAction.setActionDefinitionId(ActionFactory.REDO.getId());

		IActionBars actionBars = editorSite.getActionBars();
		if (actionBars != null) {
			actionBars.setGlobalActionHandler(ActionFactory.UNDO.getId(), undoAction);
			actionBars.setGlobalActionHandler(ActionFactory.REDO.getId(), redoAction);

		}

	}

	public void createUndoSupport_(IEditorSite editorSite) {
		IUndoManager undoManager = new TextViewerUndoManager(25);
		this.setUndoManager(undoManager);
		undoManager.connect(this);
		IUndoContext undoContext;
		if (undoManager instanceof IUndoManagerExtension) {
			undoContext = ((IUndoManagerExtension)undoManager).getUndoContext();

			OperationHistoryActionHandler undoAction = new UndoActionHandler(editorSite, undoContext);
			undoAction.setActionDefinitionId(IWorkbenchActionDefinitionIds.UNDO);

			// Create the redo action.
			OperationHistoryActionHandler redoAction = new RedoActionHandler(editorSite, undoContext);
			redoAction.setActionDefinitionId(IWorkbenchActionDefinitionIds.REDO);

			IActionBars actionBars = editorSite.getActionBars();
			if (actionBars != null) {
				actionBars.setGlobalActionHandler(IWorkbenchActionDefinitionIds.UNDO, undoAction);
				actionBars.setGlobalActionHandler(IWorkbenchActionDefinitionIds.REDO, redoAction);

			}
		}
	}

}

