/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name.handler;

import org.eclipse.core.expressions.PropertyTester;
import org.eclipse.jface.viewers.IStructuredSelection;

import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.cdm.persistence.dto.UuidAndTitleCache;
import eu.etaxonomy.taxeditor.editor.name.e4.TaxonNameEditorE4;
import eu.etaxonomy.taxeditor.event.EventUtility;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * <p>NameEditorMenuPropertyTester class.</p>
 *
 * @author n.hoffmann
 * @created Jun 22, 2010
 * @version 1.0
 */
public class NameEditorMenuPropertyTester extends PropertyTester {

	private static final String ACCEPTED = "isAcceptedTaxon"; //$NON-NLS-1$
	private static final String NOT_ACCEPTED = "isNotAcceptedTaxon"; //$NON-NLS-1$
	private static final String SYNONYM = "isSynonym"; //$NON-NLS-1$
	private static final String NOT_SYNONYM = "isNotSynonym"; //$NON-NLS-1$
	private static final String MISAPPLICATION = "isMisapplication"; //$NON-NLS-1$
	private static final String NOT_MISAPPLICATION = "isNotMisapplication"; //$NON-NLS-1$
	private static final String PROPARTE_SYNONYM = "isProparteSynonym";//$NON-NLS-1$
	private static final String NOT_PROPARTE_SYNONYM = "isNotProparteSynonym"; //$NON-NLS-1$
	private static final String TAXONBASE = "isTaxonBase"; //$NON-NLS-1$
	private static final String CONCEPT = "isConceptRelation"; //$NON-NLS-1$
	private static final String EMPTY_NAMES = "hasEmptyNames"; //$NON-NLS-1$
	private static final String ACCEPTED_AND_NO_HOMOTYPIC_SYNONYMS = "isAcceptedAndHasNoHomotypicSynonyms"; //$NON-NLS-1$
	private static final String NOT_HOMOTYPIC_SYNONYM_OF_ACCEPTED = "isNotHomotypicSynonymOfAcceptedTaxon"; //$NON-NLS-1$
	private static final String SYNONYM_WITH_HOMOTYPIC_SYNONYMS = "isSynonymInHomotypicalGroupWithMoreSynonyms"; //$NON-NLS-1$
	private static final String IS_ORPHANED = "isOrphaned";
	private static final String IS_NOT_INVALID_DESIGNATION = "isNotInvalidDesignation";//$NON-NLS-1$

	/**
	 * <p>Constructor for NameEditorMenuPropertyTester.</p>
	 */
	public NameEditorMenuPropertyTester() {
	}

	/** {@inheritDoc} */
	@Override
    public boolean test(Object receiver, String property, Object[] args,
			Object expectedValue) {

		if(receiver instanceof IStructuredSelection){

			IStructuredSelection selection = (IStructuredSelection) receiver;

			Object selectedElement = selection.getFirstElement();
			if(selectedElement instanceof UuidAndTitleCache){
			    UuidAndTitleCache uuidAndTitleCache = (UuidAndTitleCache<?>)selectedElement;
			    CdmBase cdmBase = CdmStore.getCommonService().find(uuidAndTitleCache.getType(), uuidAndTitleCache.getUuid());
			    selectedElement = cdmBase;
			}
            if(EMPTY_NAMES.equals(property)){
                return hasEmptyNames(receiver);
            }
            else if(!(selectedElement instanceof TaxonBase)){
                return false;
            }
            else if(NOT_ACCEPTED.equals(property)){
                return !isAccepted(selectedElement);
            }
			else if(ACCEPTED.equals(property)){
				return isAccepted(selectedElement);
			}
			else if(SYNONYM.equals(property)){
				return isSynonym(selectedElement);
			}
			else if(NOT_SYNONYM.equals(property)){
                return isNotSynonym(selectedElement);
            }
			else if(MISAPPLICATION.equals(property)){
				return isMisapplication(selectedElement);
			}
			else if(PROPARTE_SYNONYM.equals(property)){
                return isProparteSynonym(selectedElement);
            }
			else if(NOT_PROPARTE_SYNONYM.equals(property)){
                return isNotProparteSynonym(selectedElement);
            }
			else if(NOT_MISAPPLICATION.equals(property)){
                return isNotMisapplication(selectedElement);
            }
			else if(TAXONBASE.equals(property)){
				return isTaxonBase(selectedElement);
			}
			else if(CONCEPT.equals(property)){
				return isRelatedConcept(selectedElement);
			}
			else if(ACCEPTED_AND_NO_HOMOTYPIC_SYNONYMS.equals(property)){
				return isAcceptedAndHasNoHomotypicSynonyms(selectedElement);
			}
			else if (NOT_HOMOTYPIC_SYNONYM_OF_ACCEPTED.equals(property)){
				return isNotHomotypicSynonymOfAcceptedTaxon(selectedElement);
			}
			else if (SYNONYM_WITH_HOMOTYPIC_SYNONYMS.equals(property)){
				return isSynonymInHomotypicalGroupWithMoreSynonyms(selectedElement);
			}
			else if (IS_ORPHANED.equals(property)){
			    return isOrphaned(selectedElement);
			}
			else if (IS_NOT_INVALID_DESIGNATION.equals(property)){
                return isNotInvalidDesignation(selectedElement);
            }
		}
		return false;

	}

	public static boolean isAcceptedAndHasNoHomotypicSynonyms(Object selectedElement) {
		if (isAccepted(selectedElement)){
			Taxon taxon = (Taxon) selectedElement;
			return taxon.getSynonymsInGroup(taxon.getHomotypicGroup()).isEmpty();
		}
		return false;
	}
	public static boolean isNotAcceptedTaxon(Object selectedElement) {

        return !isAccepted(selectedElement);
    }

	public static boolean isNotMisapplication(Object selectedElement) {

        return !isMisapplication(selectedElement);
    }

	public static boolean isSynonymInHomotypicalGroupWithMoreSynonyms(Object selectedElement) {
		if (isSynonym(selectedElement)){
			Synonym synonym = (Synonym) selectedElement;
			return synonym.getHomotypicGroup().getTypifiedNames().size()>1;
		}
		return false;
	}


	public static boolean isNotHomotypicSynonymOfAcceptedTaxon(Object selectedElement) {
		if (isSynonym(selectedElement)){
			Synonym synonym = (Synonym) selectedElement;
			Taxon taxon = synonym.getAcceptedTaxon();
			if (taxon != null &&
					taxon.getHomotypicGroup().equals(synonym.getHomotypicGroup())){
				return false;
			}
		}
		return true;
	}


	/**
	 * @param receiver
	 * @return
	 */
	public static boolean hasEmptyNames(Object receiver) {
		TaxonNameEditorE4 editor = (TaxonNameEditorE4) EventUtility.getTaxonEditor();
		return editor == null ? false : editor.checkForEmptyNames();
	}

	public static boolean isRelatedConcept(Object selectedElement) {
		if(selectedElement instanceof Taxon && ((Taxon) selectedElement).isRelatedConcept() && !((TaxonNameEditorE4) EventUtility.getTaxonEditor()).getTaxon().equals(selectedElement)){
			return true;
		}
		return false;
	}

	public static boolean isTaxonBase(Object selectedElement) {
		return (selectedElement instanceof TaxonBase) ? true : false;
	}

	public static boolean isMisapplication(Object selectedElement) {
		if(selectedElement instanceof Taxon && ((Taxon) selectedElement).isMisapplication() && !((TaxonNameEditorE4) EventUtility.getTaxonEditor()).getTaxon().equals(selectedElement)){
			return true;
		}
		return false;
	}

	public static boolean isProparteSynonym(Object selectedElement) {
        if(selectedElement instanceof Taxon && ((Taxon) selectedElement).isProparteSynonym() && !((TaxonNameEditorE4) EventUtility.getTaxonEditor()).getTaxon().equals(selectedElement)){
            return true;
        }
        return false;
    }
	public static boolean isNotProparteSynonym(Object selectedElement) {
	    if (selectedElement instanceof Synonym){
	        return true;
	    }
        if(((Taxon) selectedElement).isProparteSynonym() && !((TaxonNameEditorE4) EventUtility.getTaxonEditor()).getTaxon().equals(selectedElement)){
            return false;
        }
        return true;
    }

	public static boolean isSynonym(Object selectedElement) {
		return (selectedElement instanceof Synonym) ? true : false;
	}
	public static boolean isNotSynonym(Object selectedElement) {
        return !isSynonym(selectedElement);
    }

	public static boolean isAccepted(Object selectedElement) {
	    TaxonNameEditorE4 editor = (TaxonNameEditorE4) EventUtility.getTaxonEditor();
	    Taxon accepted = editor.getTaxon();
		return (selectedElement instanceof Taxon  && accepted.equals(selectedElement)) ? true : false;
	}

	public static boolean isOrphaned(Object selectedElement) {
	    if(selectedElement instanceof Taxon){
	        return ((Taxon) selectedElement).isOrphaned();
	    }
	    else if(selectedElement instanceof Synonym){
	        return ((Synonym) selectedElement).isOrphaned();
	    }
	    return false;
	}

    /**
     * @param selectedElement
     * @return
     */
    public static boolean isProParteSynonym(Object selectedElement) {
        if(selectedElement instanceof Taxon && ((Taxon) selectedElement).isProparteSynonym()){
            return true;
        }
        return false;
    }

    /**
     * @param selectedElement
     * @return
     */
    public static boolean isNotInvalidDesignation(Object selectedElement) {
        if (selectedElement instanceof Synonym){
            return true;
        }
        if(((Taxon) selectedElement).isInvalidDesignation() ){
            return false;
        }
        return true;
    }

}
