package eu.etaxonomy.taxeditor.editor.validation;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import javax.xml.validation.Validator;

import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;

import eu.etaxonomy.cdm.model.validation.EntityConstraintViolation;
import eu.etaxonomy.cdm.model.validation.EntityValidation;
import eu.etaxonomy.cdm.model.validation.Severity;

/**
 * A class responsible for refreshing problem markers coming from the CVI
 * (Cdmlib Validation Infrastructure).
 *
 * @author ayco_holleman
 *
 */

/*
 * See following for problems with icons in Problems view
 * http://stackoverflow.com
 * /questions/13497258/markers-view-shows-my-marker-with-
 * a-red-square-insted-of-error-warning-icon
 * http://stackoverflow.com/questions/2888207
 * /eclipse-plugin-custom-icon-for-a-marker
 * http://www.eclipse.org/articles/Article-Mark%20My%20Words/mark-my-words.html
 * http
 * ://cubussapiens.hu/2010/11/markers-and-annotations-in-eclipse-for-error-feedback
 * /
 *
 * See here for difficulty of attaching markers to non-resources (files,
 * folders):
 * http://stackoverflow.com/questions/12493179/eclipse-virtual-resources
 */
public class MarkerManager {

    public static final String MARKER_TYPE_ID = "eu.etaxonomy.taxeditor.markers.validationerror"; //$NON-NLS-1$

    /**
     * The primary key (id) of the EntityConstraintViolation record
     */
    public static final String ATTRIB_DATABASE_ID = "databaseId"; //$NON-NLS-1$

    // The values of the following constants must correspond to the attributes
    // defined for the org.eclipse.core.resources.markers extension point in
    // plugin.xml

    /**
     * A user-friendly description of the type of the entity
     */
    public static final String ATTRIB_USER_FRIENDLY_TYPE_NAME = "userFriendlyTypeName"; //$NON-NLS-1$
    /**
     * A user-friendly description of the entity
     */
    public static final String ATTRIB_USER_FRIENDLY_DESCRIPTION = "userFriendlyDescription"; //$NON-NLS-1$
    /**
     * The field whose value violated a constraint
     */
    public static final String ATTRIB_USER_FRIENDLY_FIELD_NAME = "userFriendlyFieldName"; //$NON-NLS-1$
    /**
     * The value violating a constraint
     */
    public static final String ATTRIB_INVALID_VALUE = "invalidValue"; //$NON-NLS-1$
    /**
     * The message from the {@link Validator} about what was wrong.
     */
    public static final String ATTRIB_VALIDATOR_MESSAGE = "validatorMessage"; //$NON-NLS-1$
    /**
     * The class of the {@link Validator} coding for the constraint
     */
    public static final String ATTRIB_VALIDATOR_CLASS = "validatorClass"; //$NON-NLS-1$
    /**
     * The class of the validated entity
     */
    public static final String ATTRIB_ENTITY_CLASS = "entityClass"; //$NON-NLS-1$
    /**
     * The id of the validated entity
     */
    public static final String ATTRIB_ENTITY_ID = "entityId"; //$NON-NLS-1$

    private final List<EntityConstraintViolation> problems;

    MarkerManager(List<EntityValidation> results) {
        this.problems = new ArrayList<EntityConstraintViolation>();
        for (EntityValidation result : results) {
            Set<EntityConstraintViolation> problemsPerEntity = result.getEntityConstraintViolations();
            for (EntityConstraintViolation problem : problemsPerEntity) {
                problem.setEntityValidation(result);
                problems.add(problem);
            }
        }
        //MessagingUtils.info("Number of validation errors: " + problems.size());
    }

    /**
     * Delete all markers that refer to errors that do not exist any longer
     * (i.e. the corresponding database record has been deleted).
     *
     * @return The number of deleted markers
     *
     * @throws CoreException
     */
    int deleteObsoleteMarkers() throws CoreException {
        int i = 0;
        IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
        IMarker[] markers = root.findMarkers(MARKER_TYPE_ID, true, IResource.DEPTH_INFINITE);
        for (IMarker marker : markers) {
            if (isObsoleteMarker(marker)) {
                ++i;
                marker.delete();
            }
        }
        //MessagingUtils.info("Obsolete markers: " + i);
        return i;
    }

    /**
     * Create markers for new errors (i.e. no marker has been created for them
     * yet).
     *
     * @return The number of new markers
     *
     * @throws CoreException
     */
    int createMarkers() throws CoreException {
        int i = 0;
        IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
        IMarker[] markers = root.findMarkers(MARKER_TYPE_ID, true, IResource.DEPTH_INFINITE);
        for (EntityConstraintViolation problem : problems) {
            if (isNewProblem(problem, markers)) {
                ++i;
                IMarker marker = root.createMarker(MARKER_TYPE_ID);
                if (problem.getSeverity() == Severity.ERROR) {
                    marker.setAttribute(IMarker.SEVERITY, IMarker.SEVERITY_ERROR);
                } else if (problem.getSeverity() == Severity.WARNING) {
                    marker.setAttribute(IMarker.SEVERITY, IMarker.SEVERITY_WARNING);
                } else {
                    marker.setAttribute(IMarker.SEVERITY, IMarker.SEVERITY_INFO);
                }
                EntityValidation result = problem.getEntityValidation();
                marker.setAttribute(IMarker.MESSAGE, problem.getMessage());
                marker.setAttribute(ATTRIB_DATABASE_ID, new Integer(problem.getId()));
                marker.setAttribute(ATTRIB_USER_FRIENDLY_TYPE_NAME, result.getUserFriendlyTypeName());
                marker.setAttribute(ATTRIB_USER_FRIENDLY_DESCRIPTION, result.getUserFriendlyDescription());
                marker.setAttribute(ATTRIB_USER_FRIENDLY_FIELD_NAME, problem.getUserFriendlyFieldName());
                marker.setAttribute(ATTRIB_VALIDATOR_MESSAGE, problem.getMessage());
                marker.setAttribute(ATTRIB_INVALID_VALUE, problem.getInvalidValue());
                marker.setAttribute(ATTRIB_VALIDATOR_CLASS, problem.getValidator());
                marker.setAttribute(ATTRIB_ENTITY_CLASS, result.getValidatedEntityClass());
                marker.setAttribute(ATTRIB_ENTITY_ID, result.getValidatedEntityId());
            }
        }
        //MessagingUtils.info("New problems: " + i);
        return i;
    }

    private boolean isObsoleteMarker(IMarker marker) throws CoreException {
        for (EntityConstraintViolation problem : problems) {
            if (isMarkerForProblem(marker, problem)) {
                return false;
            }
        }
        return true;
    }

    private static boolean isNewProblem(EntityConstraintViolation problem, IMarker[] markers) throws CoreException {
        for (IMarker marker : markers) {
            if (isMarkerForProblem(marker, problem)) {
                return false;
            }
        }
        return true;
    }

    private static boolean isMarkerForProblem(IMarker marker, EntityConstraintViolation problem) throws CoreException {
        if (isEqual(marker, ATTRIB_DATABASE_ID, new Integer(problem.getId()))) {
            return true;
        }
        EntityValidation result = problem.getEntityValidation();
        if (!isEqual(marker, ATTRIB_ENTITY_ID, result.getValidatedEntityId())) {
            return false;
        }
        if (!isEqual(marker, ATTRIB_INVALID_VALUE, problem.getInvalidValue())) {
            return false;
        }
        if (!isEqual(marker, ATTRIB_ENTITY_CLASS, result.getValidatedEntityClass())) {
            return false;
        }
        if (!isEqual(marker, ATTRIB_USER_FRIENDLY_FIELD_NAME, problem.getUserFriendlyFieldName())) {
            return false;
        }
        if (!isEqual(marker, ATTRIB_VALIDATOR_CLASS, problem.getValidator())) {
            return false;
        }
        return true;
    }

    private static boolean isEqual(IMarker marker, String attribute, Object value) throws CoreException {
        Object val = marker.getAttribute(attribute);
        boolean equal;
        if (val == null) {
            equal = value == null;
        } else {
            equal = value != null && val.equals(value);
        }
        return equal;
    }

}
