/**
* Copyright (C) 2013 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.view.checklist;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.UUID;

import org.eclipse.ui.IMemento;
import org.eclipse.ui.IPersistable;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.service.ITaxonNodeService;
import eu.etaxonomy.cdm.api.service.ITaxonService;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.taxon.Classification;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.editor.CdmEntitySessionInput;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 *
 * @author a.oppermann
 * @date 25.04.2014
 *
 */
public class ChecklistEditorInput extends CdmEntitySessionInput implements IPersistable {

    /**
     * The selected classification
     */
    private TaxonNode taxonNode = null;

    private List<TaxonBase> taxa = new ArrayList();

    private Classification classification = null;

    private final ConversationHolder conversation;

    private List<TaxonBase> taxaToSave = new ArrayList();


    /**
     * Creates an editor input for the {@link ChecklistView} with the currently selected TaxonNode
     * and its childNodes.
     * @param taxonNode
     */
    public ChecklistEditorInput(TaxonNode taxonNode){
        super(false);
        this.conversation = CdmStore.createConversation();
        initSession();
        this.taxonNode =CdmStore.getService(ITaxonNodeService.class).load(taxonNode.getUuid(), getTaxonNodePropertyPaths());
        getChildTaxa(taxonNode);
        classification = taxonNode.getClassification();

    }


    /**
     * @param taxonNode2
     */
    private void getChildTaxa(TaxonNode taxonNode2) {
       taxonNode2.removeNullValueFromChildren();
       if (taxonNode2.hasChildNodes()){
           for (TaxonNode node: taxonNode2.getChildNodes()){
        	   node = CdmStore.getService(ITaxonNodeService.class).load(node.getUuid(), getTaxonNodePropertyPaths());
               taxa.add(HibernateProxyHelper.deproxy(node.getTaxon(), Taxon.class));
               getChildTaxa(node);
           }
       }


    }

    /**
     * Creates an editor input for the {@link ChecklistView} with the currently selected classification.
     * @param classificationUuid
     */
    public ChecklistEditorInput(Classification classification) {
        super(true);
        this.conversation = CdmStore.createConversation();
        initSession();
        this.classification = CdmStore.getCurrentApplicationConfiguration().getClassificationService().load(classification.getUuid());
        getChildTaxa(classification.getRootNode());

    }



    public List<TaxonBase> getTaxa() {
		return taxa;
	}

    public String getName() {
        if(taxonNode != null && taxonNode.getTaxon()!=null){
            return taxonNode.getTaxon().getName().getTitleCache();
        }else{
            return classification.getTitleCache();
        }
    }


    /**
     * @return the taxonNode
     */
    public TaxonNode getTaxonNode() {
        return taxonNode;
    }

    /**
     * @return classification
     */
    public Classification getClassification(){
    	return classification;
    }

//    /**
//     * @return the conversationHolder
//     */
//    public ConversationHolder getConversationHolder() {
//        return conversationHolder;
//    }



//    /* (non-Javadoc)
//     * @see java.lang.Object#hashCode()
//     */
//    @Override
//    public int hashCode() {
//        final int prime = 31;
//        int result = 1;
//        result = prime * result + ((classification == null) ? 0 : classification.hashCode());
//        result = prime * result + ((taxonNode == null) ? 0 : taxonNode.hashCode());
//        return result;
//    }

    /* (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        ChecklistEditorInput other = (ChecklistEditorInput) obj;
        if (classification == null) {
            if (other.classification != null) {
                return false;
            }
        } else if (!classification.equals(other.classification)) {
            return false;
        }
        if (taxonNode == null) {
            if (other.taxonNode != null) {
                return false;
            }
        } else if (!taxonNode.equals(other.taxonNode)) {
            return false;
        }
        return true;
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.IPersistable#saveState(org.eclipse.ui.IMemento)
     */
    @Override
    public void saveState(IMemento memento) {
        UUID uuid = classification.getUuid();
    }

    @Override
    public void merge() {
        if (!getCdmEntitySession().isActive()){
            getCdmEntitySession().bind();
        }
        CdmStore.getService(ITaxonService.class).merge(taxaToSave, true);
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.session.ICdmEntitySessionEnabled#getRootEntities()
     */
    @Override
    public List<TaxonNode> getRootEntities() {
        return Arrays.asList(taxonNode);
    }

    /**
     * @return the conversation
     */
    public ConversationHolder getConversation() {
        return conversation;
    }

    private List<String> getTaxonNodePropertyPaths() {
        List<String> taxonNodePropertyPaths = new ArrayList<String>();
        for(String propertyPath : getTaxonBasePropertyPaths()) {
            taxonNodePropertyPaths.add("taxon." + propertyPath); //$NON-NLS-1$
        }
        return taxonNodePropertyPaths;
    }

    private List<String> getTaxonBasePropertyPaths() {
        List<String> taxonBasePropertyPaths = Arrays.asList(new String[] {
                "sec", //$NON-NLS-1$
                "rights", //$NON-NLS-1$
                "sources", //$NON-NLS-1$
                "descriptions", //$NON-NLS-1$
                "descriptions.descriptionElements.feature", //$NON-NLS-1$
                "descriptions.descriptionElements.area", //$NON-NLS-1$
                "descriptions.descriptionElements.status", //$NON-NLS-1$
                "descriptions.markers", //$NON-NLS-1$

        });

        return taxonBasePropertyPaths;
    }


	public void setTaxa(List<TaxonBase> taxonList) {
		this.taxa = taxonList;

	}


    /**
     * @param element
     */
    public void addTaxonToSave(Taxon element) {
        this.taxaToSave.add(element);

    }

//    @Override
//    public void dispose(){
//    	conversation.unbind();
//    	conversation.close();
//    	getCdmEntitySession().dispose();
//    	super.dispose();
//    }

}
