// $Id$
/**
* Copyright (C) 2017 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.view.descriptive.e4;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.PostConstruct;
import javax.inject.Inject;

import org.eclipse.e4.core.contexts.ContextInjectionFactory;
import org.eclipse.e4.core.contexts.IEclipseContext;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.di.UIEventTopic;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.services.EMenuService;
import org.eclipse.jface.viewers.AbstractTreeViewer;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeNode;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Tree;

import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.IDescribable;
import eu.etaxonomy.cdm.model.name.TaxonName;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.taxeditor.bulkeditor.e4.BulkEditorE4;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.editor.name.e4.TaxonNameEditorE4;
import eu.etaxonomy.taxeditor.editor.view.concept.e4.ConceptViewPartE4;
import eu.etaxonomy.taxeditor.editor.view.descriptive.DescriptionElementDragListener;
import eu.etaxonomy.taxeditor.editor.view.descriptive.DescriptionElementDropAdapter;
import eu.etaxonomy.taxeditor.editor.view.descriptive.DescriptionElementTransfer;
import eu.etaxonomy.taxeditor.editor.view.descriptive.DescriptiveContentProvider;
import eu.etaxonomy.taxeditor.editor.view.descriptive.DescriptiveLabelProvider;
import eu.etaxonomy.taxeditor.editor.view.descriptive.DescriptiveViewerSorter;
import eu.etaxonomy.taxeditor.editor.view.descriptive.NameDescriptionFilter;
import eu.etaxonomy.taxeditor.editor.view.media.e4.MediaViewPartE4;
import eu.etaxonomy.taxeditor.event.WorkbenchEventConstants;
import eu.etaxonomy.taxeditor.model.FeatureNodeContainer;
import eu.etaxonomy.taxeditor.model.FeatureNodeContainerTree;
import eu.etaxonomy.taxeditor.model.IPartContentHasDetails;
import eu.etaxonomy.taxeditor.model.IPartContentHasFactualData;
import eu.etaxonomy.taxeditor.model.IPartContentHasSupplementalData;
import eu.etaxonomy.taxeditor.session.ICdmEntitySession;
import eu.etaxonomy.taxeditor.session.ICdmEntitySessionEnabled;
import eu.etaxonomy.taxeditor.view.e4.AbstractCdmEditorPartE4;
import eu.etaxonomy.taxeditor.view.e4.details.DetailsPartE4;
import eu.etaxonomy.taxeditor.view.e4.supplementaldata.SupplementalDataPartE4;
import eu.etaxonomy.taxeditor.workbench.part.ICollapsableExpandable;

/**
 * @author pplitzner
 * @date 15.08.2017
 *
 */
public class FactualDataPartE4 extends AbstractCdmEditorPartE4
        implements IPartContentHasDetails, IPartContentHasSupplementalData, ICdmEntitySessionEnabled,
        ICollapsableExpandable {

    /**
     * Maps {@link FeatureNodeContainerTree} to their corresponding {@link TaxonDescritpion}.<br>
     * This serves as input for the {@link ITreeContentProvider} of the {@link TreeViewer}
     */
    protected Map<DescriptionBase<?>, FeatureNodeContainerTree> featureNodeContainerCache = new HashMap<DescriptionBase<?>, FeatureNodeContainerTree>();

    protected int dndOperations = DND.DROP_COPY | DND.DROP_MOVE;

    private DescriptiveContentProvider provider;

    @PostConstruct
    public void create(Composite parent, EMenuService menuService, MPart thisPart,
            IEclipseContext context) {
        TreeViewer treeViewer = new TreeViewer(new Tree(parent, SWT.H_SCROLL
                | SWT.V_SCROLL | SWT.FULL_SELECTION | SWT.MULTI));

        this.thisPart = thisPart;

        provider = new DescriptiveContentProvider(featureNodeContainerCache);
        treeViewer.setContentProvider(provider);
        treeViewer.setLabelProvider(new DescriptiveLabelProvider());
        treeViewer.setSorter(new DescriptiveViewerSorter());
        treeViewer.setFilters(new NameDescriptionFilter());
        treeViewer.setAutoExpandLevel(2);
        Transfer[] transfers = new Transfer[] { DescriptionElementTransfer.getInstance() };
        treeViewer.addDragSupport(dndOperations, transfers, new DescriptionElementDragListener(
                this));


        viewer = treeViewer;

        // Propagate selection from viewer
        selectionChangedListener = (event -> selService.setSelection(event.getSelection()));
        viewer.addSelectionChangedListener(selectionChangedListener);
        DescriptionElementDropAdapter dropListener = new DescriptionElementDropAdapter(viewer);
        ContextInjectionFactory.inject(dropListener, context);

        treeViewer.addDropSupport(dndOperations, transfers,
                dropListener);
        //create context menu
        menuService.registerContextMenu(viewer.getControl(), "eu.etaxonomy.taxeditor.editor.popupmenu.factualDataView");

    }

    @Override
    public void selectionChanged_internal(Object selection, MPart activePart, MPart thisPart) {
        if(activePart==thisPart){
            return;
        }

        Object partObject = createPartObject(activePart);



        if ((partObject instanceof DetailsPartE4 && !(selection instanceof TaxonName))|| partObject instanceof SupplementalDataPartE4
                || partObject instanceof MediaViewPartE4 || partObject instanceof ConceptViewPartE4) {
            // do not show empty page as these views are also used to edit the
            // description selected in this view
            return;
        }


        if(partObject instanceof IPartContentHasFactualData ){
            if (!viewer.getControl().isDisposed()){
                viewer.getControl().setEnabled(true);
            }
            IStructuredSelection structuredSelection = createSelection(selection);
            if(structuredSelection==null || structuredSelection.isEmpty()){
                showEmptyPage();
                return;
            }

            Object firstElement = structuredSelection.getFirstElement();
            if (partObject instanceof TaxonNameEditorE4){
                if (firstElement != ((TaxonNameEditorE4)partObject).getTaxon()){
                    showEmptyPage();
                    return;
                }
            }

            if(firstElement instanceof TreeNode){
                firstElement = ((TreeNode) firstElement).getValue();
            }
            if(firstElement instanceof IDescribable<?>
                && partObject instanceof IPartContentHasFactualData
                    && !(firstElement instanceof SpecimenOrObservationBase && partObject instanceof BulkEditorE4)){
                   // && !(firstElement instanceof TaxonName && partObject instanceof BulkEditorE4)){
                if (firstElement instanceof TaxonName){
                    thisPart.setLabel(Messages.NameFactsDataView_Lable);
                }else{
                    thisPart.setLabel(Messages.FactualDataView_Lable);
                }
                featureNodeContainerCache.clear();
                showViewer(structuredSelection, activePart, viewer);
                return;
            }
        }
        showEmptyPage();
        return;
    }

    @Override
    protected String getViewName(){
        return Messages.DescriptiveViewPart_FACTUAL_DATA;
    }

    /** {@inheritDoc} */
    @Override
    public void changed(Object object) {
        for(FeatureNodeContainerTree containerTree : featureNodeContainerCache.values()){
            containerTree.buildTree();
        }

        if(object instanceof DescriptionElementBase){
            DescriptionElementBase descriptionElement = (DescriptionElementBase) object;
            DescriptionBase description = descriptionElement.getInDescription();
            FeatureNodeContainerTree containerTree = featureNodeContainerCache.get(description);

            FeatureNodeContainer featureNodeContainer = containerTree.getFeatureNodeContainerForDescriptionElement(descriptionElement);
            ((AbstractTreeViewer) viewer).expandToLevel(featureNodeContainer, 1);

        }
        ((AbstractTreeViewer) viewer).expandToLevel(2);
        viewer.refresh();


        if(object != null){
            StructuredSelection selection = new StructuredSelection(object);
            viewer.setSelection(selection, true);
        }
        super.changed(object);
    }

    @Inject
    @Optional
    private void updateView(@UIEventTopic(WorkbenchEventConstants.SAVE_TAXON)boolean refresh){
        if(refresh){
            viewer.refresh();
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void collapse() {
        if(viewer instanceof TreeViewer) {
            ((TreeViewer) viewer).collapseAll();
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void expand() {
        if(viewer instanceof TreeViewer) {
            ((TreeViewer) viewer).expandAll();
        }
    }

    public void toggleShowOnlyIndividualAssociations(){
        provider.toggleShowOnlyIndividualAssociations();
        viewer.refresh();
    }

    @Override
    public ICdmEntitySession getCdmEntitySession() {
        if(selectionProvidingPart != null && selectionProvidingPart instanceof ICdmEntitySessionEnabled) {
            return ((ICdmEntitySessionEnabled) selectionProvidingPart).getCdmEntitySession();
        }
        return null;
    }

    @Override
    public  List<? extends CdmBase> getRootEntities() {
        return Arrays.asList((CdmBase)getViewer().getInput());
    }
    @Override
    public Map<Object, List<String>> getPropertyPathsMap() {
        return null;
    }

}