/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.view.descriptive.operation;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.model.description.CategoricalData;
import eu.etaxonomy.cdm.model.description.CommonTaxonName;
import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.Distribution;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.IndividualsAssociation;
import eu.etaxonomy.cdm.model.description.QuantitativeData;
import eu.etaxonomy.cdm.model.description.TaxonInteraction;
import eu.etaxonomy.cdm.model.description.TextData;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.operation.AbstractPostTaxonOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;

/**
 * <p>CreateDescriptionElementOperation class.</p>
 *
 * @author p.ciardelli
 * @author n.hoffmann
 * @created 05.02.2009
 * @version 1.0
 */
public class CreateDescriptionElementOperation extends AbstractPostTaxonOperation {

	/** Constant <code>ID="eu.etaxonomy.taxeditor.editor.descripti"{trunked}</code> */
	public static final String ID = "eu.etaxonomy.taxeditor.editor.view.descriptive.command.createDescriptionElement"; //$NON-NLS-1$

	private final DescriptionBase<?> description;
	private final Feature feature;
	private DescriptionElementBase element;

	/**
	 * <p>Constructor for CreateDescriptionElementOperation.</p>
	 *
	 * @param label a {@link java.lang.String} object.
	 * @param undoContext a {@link org.eclipse.core.commands.operations.IUndoContext} object.
	 * @param taxon a {@link eu.etaxonomy.cdm.model.taxon.Taxon} object.
	 * @param description a {@link DescriptionBase} object.
	 * @param feature a {@link eu.etaxonomy.cdm.model.description.Feature} object.
	 * @param postOperationEnabled a {@link eu.etaxonomy.taxeditor.operation.IPostOperationEnabled} object.
	 */
	public CreateDescriptionElementOperation(String label, IUndoContext undoContext,
	        DescriptionBase<?> description, Feature feature, IPostOperationEnabled postOperationEnabled) {
		super(label, undoContext, (Taxon)null, postOperationEnabled);

		this.description = description;
		this.feature = feature;
	}

	/**
	 * <p>Constructor for CreateDescriptionElementOperation.</p>
	 *
	 * @param undoContext a {@link org.eclipse.core.commands.operations.IUndoContext} object.
	 * @param taxon a {@link eu.etaxonomy.cdm.model.taxon.Taxon} object.
	 * @param description a {@link DescriptionBase} object.
	 * @param feature a {@link eu.etaxonomy.cdm.model.description.Feature} object.
	 * @param element a {@link eu.etaxonomy.cdm.model.description.DescriptionElementBase} object.
	 * @param postOperationEnabled a {@link eu.etaxonomy.taxeditor.operation.IPostOperationEnabled} object.
	 * @param label a {@link java.lang.String} object.
	 */
	public CreateDescriptionElementOperation(String label,
			IUndoContext undoContext, DescriptionBase<?> description, Feature feature,
			DescriptionElementBase element, IPostOperationEnabled postOperationEnabled) {
		this(label, undoContext, description, feature, postOperationEnabled);

		this.element = element;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	/** {@inheritDoc} */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

		monitor.worked(20);
		if (element == null) {
			if (feature.isSupportsCommonTaxonName()) {
				element = CommonTaxonName.NewInstance("", null); //$NON-NLS-1$
			}
			else if (feature.isSupportsDistribution()) {
				element = Distribution.NewInstance();
			}
			else if(feature.isSupportsTaxonInteraction()){
				element = TaxonInteraction.NewInstance();
			}
			else if(feature.isSupportsIndividualAssociation()){
				element = IndividualsAssociation.NewInstance();
			}
			else if(feature.isSupportsCategoricalData()){
				element = CategoricalData.NewInstance();
			}
			else if(feature.isSupportsQuantitativeData()){
				element = QuantitativeData.NewInstance();
			}
			else {
				element = TextData.NewInstance();
			}
		}

		element.setFeature(feature);
		description.addElement(element);
		monitor.worked(40);

		return postExecute(element);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	/** {@inheritDoc} */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

		description.addElement(element);

		return postExecute(element);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	/** {@inheritDoc} */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

		description.removeElement(element);

		return postExecute(null);
	}
}
